package com.anthonycicchetti.slackbot.osrs

import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.module.kotlin.readValue
import io.ktor.client.*
import io.ktor.client.request.*
import io.ktor.client.statement.*
import io.ktor.client.call.*
import io.ktor.client.engine.cio.*
import io.ktor.client.plugins.*
import kotlinx.coroutines.*
import org.slf4j.Logger
import org.slf4j.LoggerFactory

val logger: Logger by lazy { LoggerFactory.getLogger("ItemMapper") }
val mapper: ObjectMapper  by lazy { ObjectMapper() }

val client = HttpClient(CIO) {
    install(UserAgent) {
        agent = "private_slackbot - @AnthonyCicc"
    }
}

suspend fun getItemList(): List<ApiItem> {
    val response = client.get("https://prices.runescape.wiki/api/v1/osrs/mapping")
    return when (response.status.value) {
        in 200..299 -> {
            logger.info("Got item mappings")
            mapper.readValue(response.bodyAsText())
        }

        else -> {
            logger.info("Could not get item mappings")
            emptyList()
        }
    }
}

suspend fun getOsrsItemByName(itemName: String): ApiItem? =
    getItemList().firstOrNull { item: ApiItem -> item.name.lowercase() == itemName.lowercase() }

suspend fun getOsrsItemPrice(item: ApiItem): Item {
    val response = client
        .get("https://prices.runescape.wiki/api/v1/osrs/latest")
    return when (response.status.value) {
        in 200..299 -> {
            val r: Resp = mapper.readValue(response.body<String>())
            r.data.getOrDefault(item.id.toString(), Item())
        }

        else -> {
            logger.warn("Couldn't find Item with ID ${item.id}")
            Item()
        }
    }
}

private data class Resp(
    val data: Map<String, Item> = emptyMap(),
)

data class Item(
    val low: Int = 0,
    val lowTime: Long = 0L,
    val high: Int = 0,
    val highTime: Long = 0L,
)


@JsonIgnoreProperties(ignoreUnknown = true)
data class ApiItem(
    val examine: String = "Empty Examine Text",
    val id: Int = 0,
    val members: Boolean = false,
    val lowalch: Int = 0,
    val limit: Int = 0,
    val highalch: Int = 0,
    val name: String = "Empty Name",
    val value: Int = 0,
) {
}